
public class Main {
	// the method expects an instance of the Decoration type
	// but since this interface is so-called functional,
	// i.e., it contains only one abstract method,
	// instead of this instance, a method reference can be used
	public void decoreAll(String[] p, Decoration d) {
		for (int i = 0; i < p.length; i++) {
			p[i] = d.decore(p[i]);
		}
	}

	public static void main(String[] args) {
		String[] p = {"aaa", "bbb", "ccc"};
		
		Main m = new Main();

		// 1: decoration with the decore() method of the d object:
		Decoration d = new DecorationWithPlus();
		m.decoreAll(p, d::decore);

		// decoration with an anonymous class derived from the Decoration interface:
/*		m.decoreAll(p, new Decoration() {
			public String decore(String s) {
				return s + "+";
			}
		});
*/
		// 2: decoration with a lambda expression - the DecorationWithPlus class is not needed:
/*		Decoration dl = (String s) -> { return s + "+"; };
		m.decoreAll(p, dl);
*/
		// 3: tha same, but shorter - using the automatic type inference:
/*		Decoration dl = s -> { return s + "+"; };
		m.decoreAll(p, dl);
*/
		// 4: or even shorter - no return clause:
/*		Decoration dl = s -> s + "+";
		m.decoreAll(p, dl);
*/		
		// 5: decoration with a lambda expression without nameing the operation
//		m.decoreAll(p, s -> { return s + "+"; });

		// 6: the shortest variant:
//		m.decoreAll(p, s -> s + "+");
		
		for (String s : p) 	
			System.out.println(s);
	
	}
}
